/**
 * @file
 * Imported from libinet6.
 *
 * Distributed under <a href="http://www.gnu.org/licenses/gpl2.txt">GNU/GPL</a>
 */

#ifndef HIP_LIB_TOOL_LUTIL_H
#define HIP_LIB_TOOL_LUTIL_H

#include <stdint.h>
#include <stdio.h>
#include <sys/time.h>

/* moved this here from getaddrinfo.c because it's used now in
 * getendpointinfo.c too */

struct gaih_addrtuple {
    struct gaih_addrtuple *next;
    int                    family;
    char                   addr[16];
    uint32_t               scopeid;
};

void free_gaih_addrtuple(struct gaih_addrtuple *tuple);

struct listitem {
    char             data[256];
    struct listitem *next;
};

struct list {
    struct listitem *head;
};

void initlist(struct list *);
void insert(struct list *, char *data);
void destroy(struct list *);
int length(struct list *);

/**
 * Gets an item from a linked list. Gets <code>n</code>th item from a linked
 * list.
 *
 * @param ilist a pointer to a linked list.
 * @param n     the index of the item to get.
 * @return      a pointer to <code>n</code>th item in the list, or NULL if
 *              list is NULL or if there is less than @c n items in the list.
 */
char *getitem(struct list *, int n);
char *getwithoutnewline(char *buffer, int count, FILE *f);

/**
 * Breaks a string into substrings. Breaks @c string into substrings using any
 * number of blanks and/or tab characters as separators. The substrings are
 * stored in a linked list @c list in the order of occurance.
 *
 * @param string a pointer to a string that is to be broken into substrings.
 * @param list   a pointer to a linked list where to the substrings are put.
 */
void extractsubstrings(char *string, struct list *list);

int maxof(int num_args, ...);

#endif /* HIP_LIB_TOOL_LUTIL_H */
